/*
 * Copyright (C) 2008,2009 Sebastian Pölsterl
 *
 * This file is part of GNOME DVB Daemon.
 *
 * GNOME DVB Daemon is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * GNOME DVB Daemon is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNOME DVB Daemon.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <src/IDBusSchedule.h>

typedef DBusGProxy DVBIDBusScheduleDBusProxy;
typedef DBusGProxyClass DVBIDBusScheduleDBusProxyClass;
typedef struct _DBusObjectVTable _DBusObjectVTable;

struct _DBusObjectVTable {
	void (*register_object) (DBusConnection*, const char*, void*);
};



void _dvb_id_bus_schedule_dbus_unregister (DBusConnection* connection, void* user_data);
static DBusMessage* _dbus_dvb_id_bus_schedule_introspect (DVBIDBusSchedule* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_dvb_id_bus_schedule_property_get_all (DVBIDBusSchedule* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_dvb_id_bus_schedule_GetAllEvents (DVBIDBusSchedule* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_dvb_id_bus_schedule_NowPlaying (DVBIDBusSchedule* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_dvb_id_bus_schedule_Next (DVBIDBusSchedule* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_dvb_id_bus_schedule_GetName (DVBIDBusSchedule* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_dvb_id_bus_schedule_GetShortDescription (DVBIDBusSchedule* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_dvb_id_bus_schedule_GetExtendedDescription (DVBIDBusSchedule* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_dvb_id_bus_schedule_GetDuration (DVBIDBusSchedule* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_dvb_id_bus_schedule_GetLocalStartTime (DVBIDBusSchedule* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_dvb_id_bus_schedule_IsRunning (DVBIDBusSchedule* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_dvb_id_bus_schedule_IsScrambled (DVBIDBusSchedule* self, DBusConnection* connection, DBusMessage* message);
DBusHandlerResult dvb_id_bus_schedule_dbus_proxy_filter (DBusConnection* connection, DBusMessage* message, void* user_data);
static guint32* dvb_id_bus_schedule_dbus_proxy_GetAllEvents (DVBIDBusSchedule* self, int* result_length1);
static guint32 dvb_id_bus_schedule_dbus_proxy_NowPlaying (DVBIDBusSchedule* self);
static guint32 dvb_id_bus_schedule_dbus_proxy_Next (DVBIDBusSchedule* self, guint32 event_id);
static char* dvb_id_bus_schedule_dbus_proxy_GetName (DVBIDBusSchedule* self, guint32 event_id);
static char* dvb_id_bus_schedule_dbus_proxy_GetShortDescription (DVBIDBusSchedule* self, guint32 event_id);
static char* dvb_id_bus_schedule_dbus_proxy_GetExtendedDescription (DVBIDBusSchedule* self, guint32 event_id);
static guint dvb_id_bus_schedule_dbus_proxy_GetDuration (DVBIDBusSchedule* self, guint32 event_id);
static guint* dvb_id_bus_schedule_dbus_proxy_GetLocalStartTime (DVBIDBusSchedule* self, guint32 event_id, int* result_length1);
static gboolean dvb_id_bus_schedule_dbus_proxy_IsRunning (DVBIDBusSchedule* self, guint32 event_id);
static gboolean dvb_id_bus_schedule_dbus_proxy_IsScrambled (DVBIDBusSchedule* self, guint32 event_id);
static void dvb_id_bus_schedule_dbus_proxy_interface_init (DVBIDBusScheduleIface* iface);
static void _vala_dbus_register_object (DBusConnection* connection, const char* path, void* object);
static void _vala_dbus_unregister_object (gpointer connection, GObject* object);

static const DBusObjectPathVTable _dvb_id_bus_schedule_dbus_path_vtable = {_dvb_id_bus_schedule_dbus_unregister, dvb_id_bus_schedule_dbus_message};
static const _DBusObjectVTable _dvb_id_bus_schedule_dbus_vtable = {dvb_id_bus_schedule_dbus_register_object};


guint32* dvb_id_bus_schedule_GetAllEvents (DVBIDBusSchedule* self, int* result_length1) {
	return DVB_ID_BUS_SCHEDULE_GET_INTERFACE (self)->GetAllEvents (self, result_length1);
}


/**
         * @returns: ID of currently running event
         */
guint32 dvb_id_bus_schedule_NowPlaying (DVBIDBusSchedule* self) {
	return DVB_ID_BUS_SCHEDULE_GET_INTERFACE (self)->NowPlaying (self);
}


/**
         * @returnns: ID of event that follows the given event
         */
guint32 dvb_id_bus_schedule_Next (DVBIDBusSchedule* self, guint32 event_id) {
	return DVB_ID_BUS_SCHEDULE_GET_INTERFACE (self)->Next (self, event_id);
}


char* dvb_id_bus_schedule_GetName (DVBIDBusSchedule* self, guint32 event_id) {
	return DVB_ID_BUS_SCHEDULE_GET_INTERFACE (self)->GetName (self, event_id);
}


char* dvb_id_bus_schedule_GetShortDescription (DVBIDBusSchedule* self, guint32 event_id) {
	return DVB_ID_BUS_SCHEDULE_GET_INTERFACE (self)->GetShortDescription (self, event_id);
}


char* dvb_id_bus_schedule_GetExtendedDescription (DVBIDBusSchedule* self, guint32 event_id) {
	return DVB_ID_BUS_SCHEDULE_GET_INTERFACE (self)->GetExtendedDescription (self, event_id);
}


guint dvb_id_bus_schedule_GetDuration (DVBIDBusSchedule* self, guint32 event_id) {
	return DVB_ID_BUS_SCHEDULE_GET_INTERFACE (self)->GetDuration (self, event_id);
}


guint* dvb_id_bus_schedule_GetLocalStartTime (DVBIDBusSchedule* self, guint32 event_id, int* result_length1) {
	return DVB_ID_BUS_SCHEDULE_GET_INTERFACE (self)->GetLocalStartTime (self, event_id, result_length1);
}


gboolean dvb_id_bus_schedule_IsRunning (DVBIDBusSchedule* self, guint32 event_id) {
	return DVB_ID_BUS_SCHEDULE_GET_INTERFACE (self)->IsRunning (self, event_id);
}


gboolean dvb_id_bus_schedule_IsScrambled (DVBIDBusSchedule* self, guint32 event_id) {
	return DVB_ID_BUS_SCHEDULE_GET_INTERFACE (self)->IsScrambled (self, event_id);
}


void _dvb_id_bus_schedule_dbus_unregister (DBusConnection* connection, void* user_data) {
}


static DBusMessage* _dbus_dvb_id_bus_schedule_introspect (DVBIDBusSchedule* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessage* reply;
	DBusMessageIter iter;
	GString* xml_data;
	char** children;
	int i;
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	xml_data = g_string_new ("<!DOCTYPE node PUBLIC \"-//freedesktop//DTD D-BUS Object Introspection 1.0//EN\" \"http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd\">\n");
	g_string_append (xml_data, "<node>\n<interface name=\"org.freedesktop.DBus.Introspectable\">\n  <method name=\"Introspect\">\n    <arg name=\"data\" direction=\"out\" type=\"s\"/>\n  </method>\n</interface>\n<interface name=\"org.freedesktop.DBus.Properties\">\n  <method name=\"Get\">\n    <arg name=\"interface\" direction=\"in\" type=\"s\"/>\n    <arg name=\"propname\" direction=\"in\" type=\"s\"/>\n    <arg name=\"value\" direction=\"out\" type=\"v\"/>\n  </method>\n  <method name=\"Set\">\n    <arg name=\"interface\" direction=\"in\" type=\"s\"/>\n    <arg name=\"propname\" direction=\"in\" type=\"s\"/>\n    <arg name=\"value\" direction=\"in\" type=\"v\"/>\n  </method>\n  <method name=\"GetAll\">\n    <arg name=\"interface\" direction=\"in\" type=\"s\"/>\n    <arg name=\"props\" direction=\"out\" type=\"a{sv}\"/>\n  </method>\n</interface>\n<interface name=\"org.gnome.DVB.Schedule\">\n  <method name=\"GetAllEvents\">\n    <arg name=\"result\" type=\"au\" direction=\"out\"/>\n  </method>\n  <method name=\"NowPlaying\">\n    <arg name=\"result\" type=\"u\" direction=\"out\"/>\n  </method>\n  <method name=\"Next\">\n    <arg name=\"event_id\" type=\"u\" direction=\"in\"/>\n    <arg name=\"result\" type=\"u\" direction=\"out\"/>\n  </method>\n  <method name=\"GetName\">\n    <arg name=\"event_id\" type=\"u\" direction=\"in\"/>\n    <arg name=\"result\" type=\"s\" direction=\"out\"/>\n  </method>\n  <method name=\"GetShortDescription\">\n    <arg name=\"event_id\" type=\"u\" direction=\"in\"/>\n    <arg name=\"result\" type=\"s\" direction=\"out\"/>\n  </method>\n  <method name=\"GetExtendedDescription\">\n    <arg name=\"event_id\" type=\"u\" direction=\"in\"/>\n    <arg name=\"result\" type=\"s\" direction=\"out\"/>\n  </method>\n  <method name=\"GetDuration\">\n    <arg name=\"event_id\" type=\"u\" direction=\"in\"/>\n    <arg name=\"result\" type=\"u\" direction=\"out\"/>\n  </method>\n  <method name=\"GetLocalStartTime\">\n    <arg name=\"event_id\" type=\"u\" direction=\"in\"/>\n    <arg name=\"result\" type=\"au\" direction=\"out\"/>\n  </method>\n  <method name=\"IsRunning\">\n    <arg name=\"event_id\" type=\"u\" direction=\"in\"/>\n    <arg name=\"result\" type=\"b\" direction=\"out\"/>\n  </method>\n  <method name=\"IsScrambled\">\n    <arg name=\"event_id\" type=\"u\" direction=\"in\"/>\n    <arg name=\"result\" type=\"b\" direction=\"out\"/>\n  </method>\n</interface>\n");
	dbus_connection_list_registered (connection, g_object_get_data ((GObject *) self, "dbus_object_path"), &children);
	for (i = 0; children[i]; i++) {
		g_string_append_printf (xml_data, "<node name=\"%s\"/>\n", children[i]);
	}
	g_string_append (xml_data, "</node>\n");
	dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &xml_data->str);
	g_string_free (xml_data, TRUE);
	return reply;
}


static DBusMessage* _dbus_dvb_id_bus_schedule_property_get_all (DVBIDBusSchedule* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessage* reply;
	DBusMessageIter iter, reply_iter, subiter;
	char* interface_name;
	const char* _tmp0;
	if (strcmp (dbus_message_get_signature (message), "s")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &reply_iter);
	dbus_message_iter_get_basic (&iter, &_tmp0);
	dbus_message_iter_next (&iter);
	interface_name = g_strdup (_tmp0);
	if (strcmp (interface_name, "org.gnome.DVB.Schedule") == 0) {
		dbus_message_iter_open_container (&reply_iter, DBUS_TYPE_ARRAY, "{sv}", &subiter);
		dbus_message_iter_close_container (&reply_iter, &subiter);
	} else {
		return NULL;
	}
	return reply;
}


static DBusMessage* _dbus_dvb_id_bus_schedule_GetAllEvents (DVBIDBusSchedule* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessageIter iter;
	GError* error;
	int result_length1;
	guint32* result;
	DBusMessage* reply;
	guint32* _tmp1;
	DBusMessageIter _tmp2;
	int _tmp3;
	error = NULL;
	if (strcmp (dbus_message_get_signature (message), "")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	result_length1 = 0;
	result = dvb_id_bus_schedule_GetAllEvents (self, &result_length1);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	_tmp1 = result;
	dbus_message_iter_open_container (&iter, DBUS_TYPE_ARRAY, "u", &_tmp2);
	for (_tmp3 = 0; _tmp3 < result_length1; _tmp3++) {
		dbus_uint32_t _tmp4;
		_tmp4 = *_tmp1;
		dbus_message_iter_append_basic (&_tmp2, DBUS_TYPE_UINT32, &_tmp4);
		_tmp1++;
	}
	dbus_message_iter_close_container (&iter, &_tmp2);
	return reply;
}


static DBusMessage* _dbus_dvb_id_bus_schedule_NowPlaying (DVBIDBusSchedule* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessageIter iter;
	GError* error;
	guint32 result;
	DBusMessage* reply;
	dbus_uint32_t _tmp5;
	error = NULL;
	if (strcmp (dbus_message_get_signature (message), "")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	result = dvb_id_bus_schedule_NowPlaying (self);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	_tmp5 = result;
	dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT32, &_tmp5);
	return reply;
}


static DBusMessage* _dbus_dvb_id_bus_schedule_Next (DVBIDBusSchedule* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessageIter iter;
	GError* error;
	guint32 event_id;
	dbus_uint32_t _tmp6;
	guint32 result;
	DBusMessage* reply;
	dbus_uint32_t _tmp7;
	error = NULL;
	if (strcmp (dbus_message_get_signature (message), "u")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	event_id = 0U;
	dbus_message_iter_get_basic (&iter, &_tmp6);
	dbus_message_iter_next (&iter);
	event_id = _tmp6;
	result = dvb_id_bus_schedule_Next (self, event_id);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	_tmp7 = result;
	dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT32, &_tmp7);
	return reply;
}


static DBusMessage* _dbus_dvb_id_bus_schedule_GetName (DVBIDBusSchedule* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessageIter iter;
	GError* error;
	guint32 event_id;
	dbus_uint32_t _tmp8;
	char* result;
	DBusMessage* reply;
	const char* _tmp9;
	error = NULL;
	if (strcmp (dbus_message_get_signature (message), "u")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	event_id = 0U;
	dbus_message_iter_get_basic (&iter, &_tmp8);
	dbus_message_iter_next (&iter);
	event_id = _tmp8;
	result = dvb_id_bus_schedule_GetName (self, event_id);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	_tmp9 = result;
	dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &_tmp9);
	return reply;
}


static DBusMessage* _dbus_dvb_id_bus_schedule_GetShortDescription (DVBIDBusSchedule* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessageIter iter;
	GError* error;
	guint32 event_id;
	dbus_uint32_t _tmp10;
	char* result;
	DBusMessage* reply;
	const char* _tmp11;
	error = NULL;
	if (strcmp (dbus_message_get_signature (message), "u")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	event_id = 0U;
	dbus_message_iter_get_basic (&iter, &_tmp10);
	dbus_message_iter_next (&iter);
	event_id = _tmp10;
	result = dvb_id_bus_schedule_GetShortDescription (self, event_id);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	_tmp11 = result;
	dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &_tmp11);
	return reply;
}


static DBusMessage* _dbus_dvb_id_bus_schedule_GetExtendedDescription (DVBIDBusSchedule* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessageIter iter;
	GError* error;
	guint32 event_id;
	dbus_uint32_t _tmp12;
	char* result;
	DBusMessage* reply;
	const char* _tmp13;
	error = NULL;
	if (strcmp (dbus_message_get_signature (message), "u")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	event_id = 0U;
	dbus_message_iter_get_basic (&iter, &_tmp12);
	dbus_message_iter_next (&iter);
	event_id = _tmp12;
	result = dvb_id_bus_schedule_GetExtendedDescription (self, event_id);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	_tmp13 = result;
	dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &_tmp13);
	return reply;
}


static DBusMessage* _dbus_dvb_id_bus_schedule_GetDuration (DVBIDBusSchedule* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessageIter iter;
	GError* error;
	guint32 event_id;
	dbus_uint32_t _tmp14;
	guint result;
	DBusMessage* reply;
	dbus_uint32_t _tmp15;
	error = NULL;
	if (strcmp (dbus_message_get_signature (message), "u")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	event_id = 0U;
	dbus_message_iter_get_basic (&iter, &_tmp14);
	dbus_message_iter_next (&iter);
	event_id = _tmp14;
	result = dvb_id_bus_schedule_GetDuration (self, event_id);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	_tmp15 = result;
	dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT32, &_tmp15);
	return reply;
}


static DBusMessage* _dbus_dvb_id_bus_schedule_GetLocalStartTime (DVBIDBusSchedule* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessageIter iter;
	GError* error;
	guint32 event_id;
	dbus_uint32_t _tmp16;
	int result_length1;
	guint* result;
	DBusMessage* reply;
	guint* _tmp17;
	DBusMessageIter _tmp18;
	int _tmp19;
	error = NULL;
	if (strcmp (dbus_message_get_signature (message), "u")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	event_id = 0U;
	dbus_message_iter_get_basic (&iter, &_tmp16);
	dbus_message_iter_next (&iter);
	event_id = _tmp16;
	result_length1 = 0;
	result = dvb_id_bus_schedule_GetLocalStartTime (self, event_id, &result_length1);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	_tmp17 = result;
	dbus_message_iter_open_container (&iter, DBUS_TYPE_ARRAY, "u", &_tmp18);
	for (_tmp19 = 0; _tmp19 < result_length1; _tmp19++) {
		dbus_uint32_t _tmp20;
		_tmp20 = *_tmp17;
		dbus_message_iter_append_basic (&_tmp18, DBUS_TYPE_UINT32, &_tmp20);
		_tmp17++;
	}
	dbus_message_iter_close_container (&iter, &_tmp18);
	return reply;
}


static DBusMessage* _dbus_dvb_id_bus_schedule_IsRunning (DVBIDBusSchedule* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessageIter iter;
	GError* error;
	guint32 event_id;
	dbus_uint32_t _tmp21;
	gboolean result;
	DBusMessage* reply;
	dbus_bool_t _tmp22;
	error = NULL;
	if (strcmp (dbus_message_get_signature (message), "u")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	event_id = 0U;
	dbus_message_iter_get_basic (&iter, &_tmp21);
	dbus_message_iter_next (&iter);
	event_id = _tmp21;
	result = dvb_id_bus_schedule_IsRunning (self, event_id);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	_tmp22 = result;
	dbus_message_iter_append_basic (&iter, DBUS_TYPE_BOOLEAN, &_tmp22);
	return reply;
}


static DBusMessage* _dbus_dvb_id_bus_schedule_IsScrambled (DVBIDBusSchedule* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessageIter iter;
	GError* error;
	guint32 event_id;
	dbus_uint32_t _tmp23;
	gboolean result;
	DBusMessage* reply;
	dbus_bool_t _tmp24;
	error = NULL;
	if (strcmp (dbus_message_get_signature (message), "u")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	event_id = 0U;
	dbus_message_iter_get_basic (&iter, &_tmp23);
	dbus_message_iter_next (&iter);
	event_id = _tmp23;
	result = dvb_id_bus_schedule_IsScrambled (self, event_id);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	_tmp24 = result;
	dbus_message_iter_append_basic (&iter, DBUS_TYPE_BOOLEAN, &_tmp24);
	return reply;
}


DBusHandlerResult dvb_id_bus_schedule_dbus_message (DBusConnection* connection, DBusMessage* message, void* object) {
	DBusMessage* reply;
	reply = NULL;
	if (dbus_message_is_method_call (message, "org.freedesktop.DBus.Introspectable", "Introspect")) {
		reply = _dbus_dvb_id_bus_schedule_introspect (object, connection, message);
	} else if (dbus_message_is_method_call (message, "org.freedesktop.DBus.Properties", "GetAll")) {
		reply = _dbus_dvb_id_bus_schedule_property_get_all (object, connection, message);
	} else if (dbus_message_is_method_call (message, "org.gnome.DVB.Schedule", "GetAllEvents")) {
		reply = _dbus_dvb_id_bus_schedule_GetAllEvents (object, connection, message);
	} else if (dbus_message_is_method_call (message, "org.gnome.DVB.Schedule", "NowPlaying")) {
		reply = _dbus_dvb_id_bus_schedule_NowPlaying (object, connection, message);
	} else if (dbus_message_is_method_call (message, "org.gnome.DVB.Schedule", "Next")) {
		reply = _dbus_dvb_id_bus_schedule_Next (object, connection, message);
	} else if (dbus_message_is_method_call (message, "org.gnome.DVB.Schedule", "GetName")) {
		reply = _dbus_dvb_id_bus_schedule_GetName (object, connection, message);
	} else if (dbus_message_is_method_call (message, "org.gnome.DVB.Schedule", "GetShortDescription")) {
		reply = _dbus_dvb_id_bus_schedule_GetShortDescription (object, connection, message);
	} else if (dbus_message_is_method_call (message, "org.gnome.DVB.Schedule", "GetExtendedDescription")) {
		reply = _dbus_dvb_id_bus_schedule_GetExtendedDescription (object, connection, message);
	} else if (dbus_message_is_method_call (message, "org.gnome.DVB.Schedule", "GetDuration")) {
		reply = _dbus_dvb_id_bus_schedule_GetDuration (object, connection, message);
	} else if (dbus_message_is_method_call (message, "org.gnome.DVB.Schedule", "GetLocalStartTime")) {
		reply = _dbus_dvb_id_bus_schedule_GetLocalStartTime (object, connection, message);
	} else if (dbus_message_is_method_call (message, "org.gnome.DVB.Schedule", "IsRunning")) {
		reply = _dbus_dvb_id_bus_schedule_IsRunning (object, connection, message);
	} else if (dbus_message_is_method_call (message, "org.gnome.DVB.Schedule", "IsScrambled")) {
		reply = _dbus_dvb_id_bus_schedule_IsScrambled (object, connection, message);
	}
	if (reply) {
		dbus_connection_send (connection, reply, NULL);
		dbus_message_unref (reply);
		return DBUS_HANDLER_RESULT_HANDLED;
	} else {
		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
	}
}


void dvb_id_bus_schedule_dbus_register_object (DBusConnection* connection, const char* path, void* object) {
	if (!g_object_get_data (object, "dbus_object_path")) {
		g_object_set_data (object, "dbus_object_path", g_strdup (path));
		dbus_connection_register_object_path (connection, path, &_dvb_id_bus_schedule_dbus_path_vtable, object);
		g_object_weak_ref (object, _vala_dbus_unregister_object, connection);
	}
}


static void dvb_id_bus_schedule_base_init (DVBIDBusScheduleIface * iface) {
	static gboolean initialized = FALSE;
	if (!initialized) {
		initialized = TRUE;
		g_type_set_qdata (DVB_TYPE_ID_BUS_SCHEDULE, g_quark_from_static_string ("DBusObjectVTable"), (void*) (&_dvb_id_bus_schedule_dbus_vtable));
	}
}


GType dvb_id_bus_schedule_get_type (void) {
	static GType dvb_id_bus_schedule_type_id = 0;
	if (dvb_id_bus_schedule_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (DVBIDBusScheduleIface), (GBaseInitFunc) dvb_id_bus_schedule_base_init, (GBaseFinalizeFunc) NULL, (GClassInitFunc) NULL, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
		dvb_id_bus_schedule_type_id = g_type_register_static (G_TYPE_INTERFACE, "DVBIDBusSchedule", &g_define_type_info, 0);
		g_type_interface_add_prerequisite (dvb_id_bus_schedule_type_id, G_TYPE_OBJECT);
	}
	return dvb_id_bus_schedule_type_id;
}


G_DEFINE_TYPE_EXTENDED (DVBIDBusScheduleDBusProxy, dvb_id_bus_schedule_dbus_proxy, DBUS_TYPE_G_PROXY, 0, G_IMPLEMENT_INTERFACE (DVB_TYPE_ID_BUS_SCHEDULE, dvb_id_bus_schedule_dbus_proxy_interface_init));
DVBIDBusSchedule* dvb_id_bus_schedule_dbus_proxy_new (DBusGConnection* connection, const char* name, const char* path) {
	DVBIDBusSchedule* self;
	char* filter;
	self = g_object_new (dvb_id_bus_schedule_dbus_proxy_get_type (), "connection", connection, "name", name, "path", path, "interface", "org.gnome.DVB.Schedule", NULL);
	dbus_connection_add_filter (dbus_g_connection_get_connection (connection), dvb_id_bus_schedule_dbus_proxy_filter, self, NULL);
	filter = g_strdup_printf ("type='signal',path='%s'", path);
	dbus_bus_add_match (dbus_g_connection_get_connection (connection), filter, NULL);
	g_free (filter);
	return self;
}


DBusHandlerResult dvb_id_bus_schedule_dbus_proxy_filter (DBusConnection* connection, DBusMessage* message, void* user_data) {
	if (dbus_message_has_path (message, dbus_g_proxy_get_path (user_data))) {
	}
	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
}


static void dvb_id_bus_schedule_dbus_proxy_dispose (GObject* self) {
	DBusGConnection *connection;
	g_object_get (self, "connection", &connection, NULL);
	dbus_connection_remove_filter (dbus_g_connection_get_connection (connection), dvb_id_bus_schedule_dbus_proxy_filter, self);
	G_OBJECT_CLASS (dvb_id_bus_schedule_dbus_proxy_parent_class)->dispose (self);
}


static void dvb_id_bus_schedule_dbus_proxy_class_init (DVBIDBusScheduleDBusProxyClass* klass) {
	G_OBJECT_CLASS (klass)->dispose = dvb_id_bus_schedule_dbus_proxy_dispose;
}


static void dvb_id_bus_schedule_dbus_proxy_init (DVBIDBusScheduleDBusProxy* self) {
}


static guint32* dvb_id_bus_schedule_dbus_proxy_GetAllEvents (DVBIDBusSchedule* self, int* result_length1) {
	DBusGConnection *_connection;
	DBusMessage *_message, *_reply;
	DBusMessageIter _iter;
	guint32* _result;
	int _result_length1;
	guint32* _tmp25;
	int _tmp25_length;
	int _tmp25_size;
	int _tmp25_length1;
	DBusMessageIter _tmp26;
	_message = dbus_message_new_method_call (dbus_g_proxy_get_bus_name ((DBusGProxy*) self), dbus_g_proxy_get_path ((DBusGProxy*) self), "org.gnome.DVB.Schedule", "GetAllEvents");
	dbus_message_iter_init_append (_message, &_iter);
	g_object_get (self, "connection", &_connection, NULL);
	_reply = dbus_connection_send_with_reply_and_block (dbus_g_connection_get_connection (_connection), _message, -1, NULL);
	dbus_g_connection_unref (_connection);
	dbus_message_unref (_message);
	dbus_message_iter_init (_reply, &_iter);
	_result_length1 = 0;
	_tmp25 = g_new (guint32, 5);
	_tmp25_length = 0;
	_tmp25_size = 4;
	_tmp25_length1 = 0;
	dbus_message_iter_recurse (&_iter, &_tmp26);
	for (; dbus_message_iter_get_arg_type (&_tmp26); _tmp25_length1++) {
		dbus_uint32_t _tmp27;
		if (_tmp25_size == _tmp25_length) {
			_tmp25_size = 2 * _tmp25_size;
			_tmp25 = g_renew (guint32, _tmp25, _tmp25_size + 1);
		}
		dbus_message_iter_get_basic (&_tmp26, &_tmp27);
		dbus_message_iter_next (&_tmp26);
		_tmp25[_tmp25_length++] = _tmp27;
	}
	_result_length1 = _tmp25_length1;
	dbus_message_iter_next (&_iter);
	_result = _tmp25;
	*result_length1 = _result_length1;
	dbus_message_unref (_reply);
	return _result;
}


static guint32 dvb_id_bus_schedule_dbus_proxy_NowPlaying (DVBIDBusSchedule* self) {
	DBusGConnection *_connection;
	DBusMessage *_message, *_reply;
	DBusMessageIter _iter;
	guint32 _result;
	dbus_uint32_t _tmp28;
	_message = dbus_message_new_method_call (dbus_g_proxy_get_bus_name ((DBusGProxy*) self), dbus_g_proxy_get_path ((DBusGProxy*) self), "org.gnome.DVB.Schedule", "NowPlaying");
	dbus_message_iter_init_append (_message, &_iter);
	g_object_get (self, "connection", &_connection, NULL);
	_reply = dbus_connection_send_with_reply_and_block (dbus_g_connection_get_connection (_connection), _message, -1, NULL);
	dbus_g_connection_unref (_connection);
	dbus_message_unref (_message);
	dbus_message_iter_init (_reply, &_iter);
	dbus_message_iter_get_basic (&_iter, &_tmp28);
	dbus_message_iter_next (&_iter);
	_result = _tmp28;
	dbus_message_unref (_reply);
	return _result;
}


static guint32 dvb_id_bus_schedule_dbus_proxy_Next (DVBIDBusSchedule* self, guint32 event_id) {
	DBusGConnection *_connection;
	DBusMessage *_message, *_reply;
	DBusMessageIter _iter;
	dbus_uint32_t _tmp29;
	guint32 _result;
	dbus_uint32_t _tmp30;
	_message = dbus_message_new_method_call (dbus_g_proxy_get_bus_name ((DBusGProxy*) self), dbus_g_proxy_get_path ((DBusGProxy*) self), "org.gnome.DVB.Schedule", "Next");
	dbus_message_iter_init_append (_message, &_iter);
	_tmp29 = event_id;
	dbus_message_iter_append_basic (&_iter, DBUS_TYPE_UINT32, &_tmp29);
	g_object_get (self, "connection", &_connection, NULL);
	_reply = dbus_connection_send_with_reply_and_block (dbus_g_connection_get_connection (_connection), _message, -1, NULL);
	dbus_g_connection_unref (_connection);
	dbus_message_unref (_message);
	dbus_message_iter_init (_reply, &_iter);
	dbus_message_iter_get_basic (&_iter, &_tmp30);
	dbus_message_iter_next (&_iter);
	_result = _tmp30;
	dbus_message_unref (_reply);
	return _result;
}


static char* dvb_id_bus_schedule_dbus_proxy_GetName (DVBIDBusSchedule* self, guint32 event_id) {
	DBusGConnection *_connection;
	DBusMessage *_message, *_reply;
	DBusMessageIter _iter;
	dbus_uint32_t _tmp31;
	char* _result;
	const char* _tmp32;
	_message = dbus_message_new_method_call (dbus_g_proxy_get_bus_name ((DBusGProxy*) self), dbus_g_proxy_get_path ((DBusGProxy*) self), "org.gnome.DVB.Schedule", "GetName");
	dbus_message_iter_init_append (_message, &_iter);
	_tmp31 = event_id;
	dbus_message_iter_append_basic (&_iter, DBUS_TYPE_UINT32, &_tmp31);
	g_object_get (self, "connection", &_connection, NULL);
	_reply = dbus_connection_send_with_reply_and_block (dbus_g_connection_get_connection (_connection), _message, -1, NULL);
	dbus_g_connection_unref (_connection);
	dbus_message_unref (_message);
	dbus_message_iter_init (_reply, &_iter);
	dbus_message_iter_get_basic (&_iter, &_tmp32);
	dbus_message_iter_next (&_iter);
	_result = g_strdup (_tmp32);
	dbus_message_unref (_reply);
	return _result;
}


static char* dvb_id_bus_schedule_dbus_proxy_GetShortDescription (DVBIDBusSchedule* self, guint32 event_id) {
	DBusGConnection *_connection;
	DBusMessage *_message, *_reply;
	DBusMessageIter _iter;
	dbus_uint32_t _tmp33;
	char* _result;
	const char* _tmp34;
	_message = dbus_message_new_method_call (dbus_g_proxy_get_bus_name ((DBusGProxy*) self), dbus_g_proxy_get_path ((DBusGProxy*) self), "org.gnome.DVB.Schedule", "GetShortDescription");
	dbus_message_iter_init_append (_message, &_iter);
	_tmp33 = event_id;
	dbus_message_iter_append_basic (&_iter, DBUS_TYPE_UINT32, &_tmp33);
	g_object_get (self, "connection", &_connection, NULL);
	_reply = dbus_connection_send_with_reply_and_block (dbus_g_connection_get_connection (_connection), _message, -1, NULL);
	dbus_g_connection_unref (_connection);
	dbus_message_unref (_message);
	dbus_message_iter_init (_reply, &_iter);
	dbus_message_iter_get_basic (&_iter, &_tmp34);
	dbus_message_iter_next (&_iter);
	_result = g_strdup (_tmp34);
	dbus_message_unref (_reply);
	return _result;
}


static char* dvb_id_bus_schedule_dbus_proxy_GetExtendedDescription (DVBIDBusSchedule* self, guint32 event_id) {
	DBusGConnection *_connection;
	DBusMessage *_message, *_reply;
	DBusMessageIter _iter;
	dbus_uint32_t _tmp35;
	char* _result;
	const char* _tmp36;
	_message = dbus_message_new_method_call (dbus_g_proxy_get_bus_name ((DBusGProxy*) self), dbus_g_proxy_get_path ((DBusGProxy*) self), "org.gnome.DVB.Schedule", "GetExtendedDescription");
	dbus_message_iter_init_append (_message, &_iter);
	_tmp35 = event_id;
	dbus_message_iter_append_basic (&_iter, DBUS_TYPE_UINT32, &_tmp35);
	g_object_get (self, "connection", &_connection, NULL);
	_reply = dbus_connection_send_with_reply_and_block (dbus_g_connection_get_connection (_connection), _message, -1, NULL);
	dbus_g_connection_unref (_connection);
	dbus_message_unref (_message);
	dbus_message_iter_init (_reply, &_iter);
	dbus_message_iter_get_basic (&_iter, &_tmp36);
	dbus_message_iter_next (&_iter);
	_result = g_strdup (_tmp36);
	dbus_message_unref (_reply);
	return _result;
}


static guint dvb_id_bus_schedule_dbus_proxy_GetDuration (DVBIDBusSchedule* self, guint32 event_id) {
	DBusGConnection *_connection;
	DBusMessage *_message, *_reply;
	DBusMessageIter _iter;
	dbus_uint32_t _tmp37;
	guint _result;
	dbus_uint32_t _tmp38;
	_message = dbus_message_new_method_call (dbus_g_proxy_get_bus_name ((DBusGProxy*) self), dbus_g_proxy_get_path ((DBusGProxy*) self), "org.gnome.DVB.Schedule", "GetDuration");
	dbus_message_iter_init_append (_message, &_iter);
	_tmp37 = event_id;
	dbus_message_iter_append_basic (&_iter, DBUS_TYPE_UINT32, &_tmp37);
	g_object_get (self, "connection", &_connection, NULL);
	_reply = dbus_connection_send_with_reply_and_block (dbus_g_connection_get_connection (_connection), _message, -1, NULL);
	dbus_g_connection_unref (_connection);
	dbus_message_unref (_message);
	dbus_message_iter_init (_reply, &_iter);
	dbus_message_iter_get_basic (&_iter, &_tmp38);
	dbus_message_iter_next (&_iter);
	_result = _tmp38;
	dbus_message_unref (_reply);
	return _result;
}


static guint* dvb_id_bus_schedule_dbus_proxy_GetLocalStartTime (DVBIDBusSchedule* self, guint32 event_id, int* result_length1) {
	DBusGConnection *_connection;
	DBusMessage *_message, *_reply;
	DBusMessageIter _iter;
	dbus_uint32_t _tmp39;
	guint* _result;
	int _result_length1;
	guint* _tmp40;
	int _tmp40_length;
	int _tmp40_size;
	int _tmp40_length1;
	DBusMessageIter _tmp41;
	_message = dbus_message_new_method_call (dbus_g_proxy_get_bus_name ((DBusGProxy*) self), dbus_g_proxy_get_path ((DBusGProxy*) self), "org.gnome.DVB.Schedule", "GetLocalStartTime");
	dbus_message_iter_init_append (_message, &_iter);
	_tmp39 = event_id;
	dbus_message_iter_append_basic (&_iter, DBUS_TYPE_UINT32, &_tmp39);
	g_object_get (self, "connection", &_connection, NULL);
	_reply = dbus_connection_send_with_reply_and_block (dbus_g_connection_get_connection (_connection), _message, -1, NULL);
	dbus_g_connection_unref (_connection);
	dbus_message_unref (_message);
	dbus_message_iter_init (_reply, &_iter);
	_result_length1 = 0;
	_tmp40 = g_new (guint, 5);
	_tmp40_length = 0;
	_tmp40_size = 4;
	_tmp40_length1 = 0;
	dbus_message_iter_recurse (&_iter, &_tmp41);
	for (; dbus_message_iter_get_arg_type (&_tmp41); _tmp40_length1++) {
		dbus_uint32_t _tmp42;
		if (_tmp40_size == _tmp40_length) {
			_tmp40_size = 2 * _tmp40_size;
			_tmp40 = g_renew (guint, _tmp40, _tmp40_size + 1);
		}
		dbus_message_iter_get_basic (&_tmp41, &_tmp42);
		dbus_message_iter_next (&_tmp41);
		_tmp40[_tmp40_length++] = _tmp42;
	}
	_result_length1 = _tmp40_length1;
	dbus_message_iter_next (&_iter);
	_result = _tmp40;
	*result_length1 = _result_length1;
	dbus_message_unref (_reply);
	return _result;
}


static gboolean dvb_id_bus_schedule_dbus_proxy_IsRunning (DVBIDBusSchedule* self, guint32 event_id) {
	DBusGConnection *_connection;
	DBusMessage *_message, *_reply;
	DBusMessageIter _iter;
	dbus_uint32_t _tmp43;
	gboolean _result;
	dbus_bool_t _tmp44;
	_message = dbus_message_new_method_call (dbus_g_proxy_get_bus_name ((DBusGProxy*) self), dbus_g_proxy_get_path ((DBusGProxy*) self), "org.gnome.DVB.Schedule", "IsRunning");
	dbus_message_iter_init_append (_message, &_iter);
	_tmp43 = event_id;
	dbus_message_iter_append_basic (&_iter, DBUS_TYPE_UINT32, &_tmp43);
	g_object_get (self, "connection", &_connection, NULL);
	_reply = dbus_connection_send_with_reply_and_block (dbus_g_connection_get_connection (_connection), _message, -1, NULL);
	dbus_g_connection_unref (_connection);
	dbus_message_unref (_message);
	dbus_message_iter_init (_reply, &_iter);
	dbus_message_iter_get_basic (&_iter, &_tmp44);
	dbus_message_iter_next (&_iter);
	_result = _tmp44;
	dbus_message_unref (_reply);
	return _result;
}


static gboolean dvb_id_bus_schedule_dbus_proxy_IsScrambled (DVBIDBusSchedule* self, guint32 event_id) {
	DBusGConnection *_connection;
	DBusMessage *_message, *_reply;
	DBusMessageIter _iter;
	dbus_uint32_t _tmp45;
	gboolean _result;
	dbus_bool_t _tmp46;
	_message = dbus_message_new_method_call (dbus_g_proxy_get_bus_name ((DBusGProxy*) self), dbus_g_proxy_get_path ((DBusGProxy*) self), "org.gnome.DVB.Schedule", "IsScrambled");
	dbus_message_iter_init_append (_message, &_iter);
	_tmp45 = event_id;
	dbus_message_iter_append_basic (&_iter, DBUS_TYPE_UINT32, &_tmp45);
	g_object_get (self, "connection", &_connection, NULL);
	_reply = dbus_connection_send_with_reply_and_block (dbus_g_connection_get_connection (_connection), _message, -1, NULL);
	dbus_g_connection_unref (_connection);
	dbus_message_unref (_message);
	dbus_message_iter_init (_reply, &_iter);
	dbus_message_iter_get_basic (&_iter, &_tmp46);
	dbus_message_iter_next (&_iter);
	_result = _tmp46;
	dbus_message_unref (_reply);
	return _result;
}


static void dvb_id_bus_schedule_dbus_proxy_interface_init (DVBIDBusScheduleIface* iface) {
	iface->GetAllEvents = dvb_id_bus_schedule_dbus_proxy_GetAllEvents;
	iface->NowPlaying = dvb_id_bus_schedule_dbus_proxy_NowPlaying;
	iface->Next = dvb_id_bus_schedule_dbus_proxy_Next;
	iface->GetName = dvb_id_bus_schedule_dbus_proxy_GetName;
	iface->GetShortDescription = dvb_id_bus_schedule_dbus_proxy_GetShortDescription;
	iface->GetExtendedDescription = dvb_id_bus_schedule_dbus_proxy_GetExtendedDescription;
	iface->GetDuration = dvb_id_bus_schedule_dbus_proxy_GetDuration;
	iface->GetLocalStartTime = dvb_id_bus_schedule_dbus_proxy_GetLocalStartTime;
	iface->IsRunning = dvb_id_bus_schedule_dbus_proxy_IsRunning;
	iface->IsScrambled = dvb_id_bus_schedule_dbus_proxy_IsScrambled;
}


static void _vala_dbus_register_object (DBusConnection* connection, const char* path, void* object) {
	const _DBusObjectVTable * vtable;
	vtable = g_type_get_qdata (G_TYPE_FROM_INSTANCE (object), g_quark_from_static_string ("DBusObjectVTable"));
	if (vtable) {
		vtable->register_object (connection, path, object);
	} else {
		g_warning ("Object does not implement any D-Bus interface");
	}
}


static void _vala_dbus_unregister_object (gpointer connection, GObject* object) {
	char* path;
	path = g_object_steal_data ((GObject*) object, "dbus_object_path");
	dbus_connection_unregister_object_path (connection, path);
	g_free (path);
}




